//=============================================================================
//
//  CLASS PinholeCamera
//
//=============================================================================
#ifndef RAYTRACER_PINHOLECAMERA_H
#define RAYTRACER_PINHOLECAMERA_H

//== INCLUDES =================================================================
#include "Types.h"
#include "Ray.h"

//== CLASS DEFINITION =========================================================

/*!
	Simple pinholecamera-simulation which defines an image- (or film) plane 
	in space. It uses Eye, LookAt and Up to define
	orientation and focal length for the imageplane's distance from the eye.

	\author Gero Mueller
*/
class   PinholeCamera
{

    //-------------------------------------------------------------- private data
private:
    VectorType3     m_v3Eye, m_v3LookAt, m_v3Up;
    double        m_rFocalLength;
    double        m_rLeft, m_rBottom, m_rRight, m_rTop;
    MatrixType33    m_m33ONB;
    void            MakeONB();

    //-------------------------------------------------------------- constructors
public:

    /*!
	  Constructor with initializing values
	*/
    PinholeCamera(
        const VectorType3   &v3Eye = VectorType3(0,  0,  0),      
        const VectorType3   &v3LookAt = VectorType3(0,  0,  -1),  
        const VectorType3   &v3Up = VectorType3(0,  1,  0),       
        const double      rFocalLength = 1,   
        const double      rLeft = -1,         
        const double      rBottom = -1,       
        const double      rRight = 1,         
        const double      rTop = 1 )  :
    m_v3Eye( v3Eye ),
    m_v3LookAt( v3LookAt ),
    m_v3Up( v3Up ),
    m_rFocalLength( rFocalLength ),
    m_rLeft( rLeft ),
    m_rBottom( rBottom ),
    m_rRight( rRight ),
    m_rTop( rTop )
    {
        MakeONB();
    };

    //----------------------------------------------------------- element access
public:

    //@{

    //!Eye-vector
    void setEye(
        const VectorType3   &v3Eye )    
    {
        m_v3Eye = v3Eye;
        MakeONB();
    };

    
    const VectorType3 &getEye() const
    {
        return m_v3Eye;
    };

    //@}
    //@{

    //!LookAt-vector
    void setLookAt(
        const VectorType3   &v3LookAt ) 
    {
        m_v3LookAt = v3LookAt;
        MakeONB();
    };

    
    const VectorType3 &getLookAt() const
    {
        return m_v3LookAt;
    };

    //@}
    //@{

    //!Up-vector
    void setUp(
        const VectorType3   &v3Up ) 
    {
        m_v3Up = v3Up;
        MakeONB();
    };

    
    const VectorType3 &getUp() const
    {
        return m_v3Up;
    };

    //@}
    //@{

    //!Focal length
    void setFocalLength(
        const double  rFocalLength )  
    {
        m_rFocalLength = rFocalLength;
    };

    
    double getFocalLength() const
    {
        return m_rFocalLength;
    };

    //@{

    //!viewing window corners
    void setWindow(
        double    rLeft,      
        double    rBottom,    
        double    rRight,     
        double    rTop )      
    {
        m_rLeft = rLeft;
        m_rBottom = rBottom;
        m_rRight = rRight;
        m_rTop = rTop;
    };

    
    void getWindow(
        double    *rLeft,     
        double    *rBottom,   
        double    *rRight,    
        double    *rTop )     
    {
        *rLeft = m_rLeft;
        *rBottom = m_rBottom;
        *rRight = m_rRight;
        *rTop = m_rTop;
    };

    //!force aspect-ratio and center imageplane
    void setAspectRatio(
        double    rAspect )   
    {
        double    r_width = fabs( m_rRight - m_rLeft );
        double    r_height = r_width / rAspect;
        m_rLeft = -r_width / 2.;
        m_rRight = r_width / 2.;
        m_rBottom = -r_height / 2.;
        m_rTop = r_height / 2.;
    };

    //@}
    //----------------------------------------------------------- functionality
    //! generateRay for pixel (x, y)
    Ray generateRay( double    x,
                     double    y,
                     double    rWidth,
                     double    rHeight ) const;

    //! set to canonical position
    void Reset()
    {
        m_v3Eye = VectorType3( 0, 0, 1 );
        m_v3LookAt = VectorType3( 0, 0, 0 );
        m_v3Up = VectorType3( 0, 1, 0 );
        m_rFocalLength = 2;

        //m_rFovy = 85;
        setWindow( -1, -1, 1, 1 );
    };
};
//=============================================================================
#endif // RAYTRACER_PINHOLECAMERA_H

